@@ -5,7 +5,7 @@
  * as per the DfuSe 1.1a specification (ST documents AN3156, AN2606)
  * The DfuSe file format is described in ST document UM0391.
  *
- * Copyright 2010-2014 Tormod Volden <debian.tormod@gmail.com>
+ * Copyright 2010-2018 Tormod Volden <debian.tormod@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include "dfu_file.h"
 #include "dfuse.h"
 #include "dfuse_mem.h"
+#include "quirks.h"
 
 #define DFU_TIMEOUT 5000
 
@@ -40,18 +41,20 @@ extern int verbose;
 static unsigned int last_erased_page = 1; /* non-aligned value, won't match */
 static struct memsegment *mem_layout;
 static unsigned int dfuse_address = 0;
+static unsigned int dfuse_address_present = 0;
 static unsigned int dfuse_length = 0;
 static int dfuse_force = 0;
 static int dfuse_leave = 0;
 static int dfuse_unprotect = 0;
 static int dfuse_mass_erase = 0;
+static int dfuse_will_reset = 0;
 
-unsigned int quad2uint(unsigned char *p)
+static unsigned int quad2uint(unsigned char *p)
 {
 	return (*p + (*(p + 1) << 8) + (*(p + 2) << 16) + (*(p + 3) << 24));
 }
 
-void dfuse_parse_options(const char *options)
+static void dfuse_parse_options(const char *options)
 {
 	char *end;
 	const char *endword;
@@ -66,6 +69,7 @@ void dfuse_parse_options(const char *options)
 		number = strtoul(options, &end, 0);
 		if (end == endword) {
 			dfuse_address = number;
+			dfuse_address_present = 1;
 		} else {
 			errx(EX_IOERR, "Invalid dfuse address: %s", options);
 		}
@@ -101,6 +105,11 @@ void dfuse_parse_options(const char *options)
 			options += 10;
 			continue;
 		}
+		if (!strncmp(options, "will-reset", endword - options)) {
+			dfuse_will_reset = 1;
+			options += 10;
+			continue;
+		}
 
 		/* any valid number is interpreted as upload length */
 		number = strtoul(options, &end, 0);
@@ -114,7 +123,7 @@ void dfuse_parse_options(const char *options)
 }
 
 /* DFU_UPLOAD request for DfuSe 1.1a */
-int dfuse_upload(struct dfu_if *dif, const unsigned short length,
+static int dfuse_upload(struct dfu_if *dif, const unsigned short length,
 		 unsigned char *data, unsigned short transaction)
 {
 	int status;
@@ -130,14 +139,14 @@ int dfuse_upload(struct dfu_if *dif, const unsigned short length,
 		 /* wLength       */	 length,
 					 DFU_TIMEOUT);
 	if (status < 0) {
-		errx(EX_IOERR, "%s: libusb_control_msg returned %d",
-			__FUNCTION__, status);
+		warnx("dfuse_upload: libusb_control_transfer returned %d (%s)",
+		      status, libusb_error_name(status));
 	}
 	return status;
 }
 
 /* DFU_DNLOAD request for DfuSe 1.1a */
-int dfuse_download(struct dfu_if *dif, const unsigned short length,
+static int dfuse_download(struct dfu_if *dif, const unsigned short length,
 		   unsigned char *data, unsigned short transaction)
 {
 	int status;
@@ -153,15 +162,15 @@ int dfuse_download(struct dfu_if *dif, const unsigned short length,
 		 /* wLength       */	 length,
 					 DFU_TIMEOUT);
 	if (status < 0) {
-		errx(EX_IOERR, "%s: libusb_control_transfer returned %d",
-			__FUNCTION__, status);
+		warnx("dfuse_download: libusb_control_transfer returned %d (%s)",
+		      status, libusb_error_name(status));
 	}
 	return status;
 }
 
 /* DfuSe only commands */
 /* Leaves the device in dfuDNLOAD-IDLE state */
-int dfuse_special_command(struct dfu_if *dif, unsigned int address,
+static int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 			  enum dfuse_command command)
 {
 	const char* dfuse_command_name[] = { "SET_ADDRESS" , "ERASE_PAGE",
@@ -171,6 +180,9 @@ int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 	int ret;
 	struct dfu_status dst;
 	int firstpoll = 1;
+	int zerotimeouts = 0;
+	int polltimeout = 0;
+	int stalls = 0;
 
 	if (command == ERASE_PAGE) {
 		struct memsegment *segment;
@@ -182,16 +194,16 @@ int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 				address);
 		}
 		page_size = segment->pagesize;
-		if (verbose > 1)
-			printf("Erasing page size %i at address 0x%08x, page "
+		if (verbose)
+			fprintf(stderr, "Erasing page size %i at address 0x%08x, page "
 			       "starting at 0x%08x\n", page_size, address,
 			       address & ~(page_size - 1));
 		buf[0] = 0x41;	/* Erase command */
 		length = 5;
 		last_erased_page = address & ~(page_size - 1);
 	} else if (command == SET_ADDRESS) {
-		if (verbose > 2)
-			printf("  Setting address pointer to 0x%08x\n",
+		if (verbose > 1)
+			fprintf(stderr, "  Setting address pointer to 0x%08x\n",
 			       address);
 		buf[0] = 0x21;	/* Set Address Pointer command */
 		length = 5;
@@ -216,26 +228,47 @@ int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 	}
 	do {
 		ret = dfu_get_status(dif, &dst);
-		if (ret < 0) {
+		/* Workaround for some STM32L4 bootloaders that report a too
+		 * short poll timeout and may stall the pipe when we poll */
+		if (ret == LIBUSB_ERROR_PIPE && polltimeout != 0 && stalls < 3) {
+			dst.bState = DFU_STATE_dfuDNBUSY;
+			stalls++;
+			if (verbose)
+				fprintf(stderr, "* Device stalled USB pipe, reusing last poll timeout\n");
+		} else if (ret < 0) {
 			errx(EX_IOERR, "Error during special command \"%s\" get_status",
 			     dfuse_command_name[command]);
+		} else {
+			polltimeout = dst.bwPollTimeout;
 		}
 		if (firstpoll) {
 			firstpoll = 0;
 			if (dst.bState != DFU_STATE_dfuDNBUSY) {
-				printf("state(%u) = %s, status(%u) = %s\n", dst.bState,
+				fprintf(stderr, "state(%u) = %s, status(%u) = %s\n", dst.bState,
 				       dfu_state_to_string(dst.bState), dst.bStatus,
 				       dfu_status_to_string(dst.bStatus));
 				errx(EX_IOERR, "Wrong state after command \"%s\" download",
 				     dfuse_command_name[command]);
 			}
+			/* STM32F405 lies about mass erase timeout */
+			if (command == MASS_ERASE && dst.bwPollTimeout == 100) {
+				polltimeout = 35000; /* Datasheet says up to 32 seconds */
+				printf("Setting timeout to 35 seconds\n");
+			}
 		}
 		/* wait while command is executed */
-		if (verbose)
-			printf("   Poll timeout %i ms\n", dst.bwPollTimeout);
-		milli_sleep(dst.bwPollTimeout);
+		if (verbose > 1)
+			fprintf(stderr, "   Poll timeout %i ms\n", polltimeout);
+		milli_sleep(polltimeout);
 		if (command == READ_UNPROTECT)
 			return ret;
+		/* Workaround for e.g. Black Magic Probe getting stuck */
+		if (dst.bwPollTimeout == 0) {
+			if (++zerotimeouts == 100)
+				errx(EX_IOERR, "Device stuck after special command request");
+		} else {
+			zerotimeouts = 0;
+		}
 	} while (dst.bState == DFU_STATE_dfuDNBUSY);
 
 	if (dst.bStatus != DFU_STATUS_OK) {
@@ -245,7 +278,7 @@ int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 	return ret;
 }
 
-int dfuse_dnload_chunk(struct dfu_if *dif, unsigned char *data, int size,
+static int dfuse_dnload_chunk(struct dfu_if *dif, unsigned char *data, int size,
 		       int transaction)
 {
 	int bytes_sent;
@@ -268,14 +301,15 @@ int dfuse_dnload_chunk(struct dfu_if *dif, unsigned char *data, int size,
 		milli_sleep(dst.bwPollTimeout);
 	} while (dst.bState != DFU_STATE_dfuDNLOAD_IDLE &&
 		 dst.bState != DFU_STATE_dfuERROR &&
-		 dst.bState != DFU_STATE_dfuMANIFEST);
+		 dst.bState != DFU_STATE_dfuMANIFEST &&
+		 !(dfuse_will_reset && (dst.bState == DFU_STATE_dfuDNBUSY)));
 
 	if (dst.bState == DFU_STATE_dfuMANIFEST)
 			printf("Transitioning to dfuMANIFEST state\n");
 
 	if (dst.bStatus != DFU_STATUS_OK) {
-		printf(" failed!\n");
-		printf("state(%u) = %s, status(%u) = %s\n", dst.bState,
+		fprintf(stderr, " failed!\n");
+		fprintf(stderr, "state(%u) = %s, status(%u) = %s\n", dst.bState,
 		       dfu_state_to_string(dst.bState), dst.bStatus,
 		       dfu_status_to_string(dst.bStatus));
 		return -1;
@@ -298,12 +332,14 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, int fd,
 		dfuse_parse_options(dfuse_options);
 	if (dfuse_length)
 		upload_limit = dfuse_length;
-	if (dfuse_address) {
+	if (dfuse_address_present) {
 		struct memsegment *segment;
 
 		mem_layout = parse_memory_layout((char *)dif->alt_name);
 		if (!mem_layout)
 			errx(EX_IOERR, "Failed to parse memory layout");
+		if (dif->quirks & QUIRK_DFUSE_LAYOUT)
+			fixup_dfuse_layout(dif, &mem_layout);
 
 		segment = find_segment(mem_layout, dfuse_address);
 		if (!dfuse_force &&
@@ -312,17 +348,25 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, int fd,
 				dfuse_address);
 
 		if (!upload_limit) {
-			upload_limit = segment->end - dfuse_address + 1;
-			printf("Limiting upload to end of memory segment, "
-			       "%i bytes\n", upload_limit);
+			if (segment) {
+				upload_limit = segment->end - dfuse_address + 1;
+				printf("Limiting upload to end of memory segment, "
+				       "%i bytes\n", upload_limit);
+			} else {
+				/* unknown segment - i.e. "force" has been used */
+				upload_limit = 0x4000;
+				printf("Limiting upload to %i bytes\n", upload_limit);
+			}
 		}
 		dfuse_special_command(dif, dfuse_address, SET_ADDRESS);
 		dfu_abort_to_idle(dif);
 	} else {
 		/* Boot loader decides the start address, unknown to us */
 		/* Use a short length to lower risk of running out of bounds */
-		if (!upload_limit)
+		if (!upload_limit) {
+			warnx("Unbound upload not supported on DfuSe devices");
 			upload_limit = 0x4000;
+		}
 		printf("Limiting default upload to %i bytes\n", upload_limit);
 	}
 
@@ -359,7 +403,8 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, int fd,
 
 	dfu_abort_to_idle(dif);
 	if (dfuse_leave) {
-		dfuse_special_command(dif, dfuse_address, SET_ADDRESS);
+		if (dfuse_address_present)
+			dfuse_special_command(dif, dfuse_address, SET_ADDRESS);
 		dfuse_dnload_chunk(dif, NULL, 0, 2); /* Zero-size */
 	}
 
@@ -371,7 +416,7 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, int fd,
 
 /* Writes an element of any size to the device, taking care of page erases */
 /* returns 0 on success, otherwise -EINVAL */
-int dfuse_dnload_element(struct dfu_if *dif, unsigned int dwElementAddress,
+static int dfuse_dnload_element(struct dfu_if *dif, unsigned int dwElementAddress,
 			 unsigned int dwElementSize, unsigned char *data,
 			 int xfer_size)
 {
@@ -382,13 +427,16 @@ int dfuse_dnload_element(struct dfu_if *dif, unsigned int dwElementAddress,
 	/* Check at least that we can write to the last address */
 	segment =
 	    find_segment(mem_layout, dwElementAddress + dwElementSize - 1);
-	if (!segment || !(segment->memtype & DFUSE_WRITEABLE)) {
+	if (!dfuse_force &&
+            (!segment || !(segment->memtype & DFUSE_WRITEABLE))) {
 		errx(EX_IOERR, "Last page at 0x%08x is not writeable",
 			dwElementAddress + dwElementSize - 1);
 	}
 
-	dfu_progress_bar("Download", 0, 1);
+	if (!verbose)
+		dfu_progress_bar("Erase   ", 0, 1);
 
+	/* First pass: Erase involved pages if needed */
 	for (p = 0; p < (int)dwElementSize; p += xfer_size) {
 		int page_size;
 		unsigned int erase_address;
@@ -396,10 +444,16 @@ int dfuse_dnload_element(struct dfu_if *dif, unsigned int dwElementAddress,
 		int chunk_size = xfer_size;
 
 		segment = find_segment(mem_layout, address);
-		if (!segment || !(segment->memtype & DFUSE_WRITEABLE)) {
+		if (!dfuse_force &&
+		    (!segment || !(segment->memtype & DFUSE_WRITEABLE))) {
 			errx(EX_IOERR, "Page at 0x%08x is not writeable",
 				address);
 		}
+		/* If the location is not in the memory map we skip erasing */
+		/* since we wouldn't know the correct page size for flash erase */
+		if (!segment)
+			continue;
+
 		page_size = segment->pagesize;
 
 		/* check if this is the last chunk */
@@ -420,17 +474,33 @@ int dfuse_dnload_element(struct dfu_if *dif, unsigned int dwElementAddress,
 
 			if (((address + chunk_size - 1) & ~(page_size - 1)) !=
 			    last_erased_page) {
-				if (verbose > 2)
-					printf(" Chunk extends into next page,"
+				if (verbose > 1)
+					fprintf(stderr, " Chunk extends into next page,"
 					       " erase it as well\n");
 				dfuse_special_command(dif,
 						      address + chunk_size - 1,
 						      ERASE_PAGE);
 			}
+			if (!verbose)
+				dfu_progress_bar("Erase   ", p, dwElementSize);
 		}
+	}
+	if (!verbose)
+		dfu_progress_bar("Erase   ", dwElementSize, dwElementSize);
+	if (!verbose)
+		dfu_progress_bar("Download", 0, 1);
+
+	/* Second pass: Write data to (erased) pages */
+	for (p = 0; p < (int)dwElementSize; p += xfer_size) {
+		unsigned int address = dwElementAddress + p;
+		int chunk_size = xfer_size;
+
+		/* check if this is the last chunk */
+		if (p + chunk_size > (int)dwElementSize)
+			chunk_size = dwElementSize - p;
 
 		if (verbose) {
-			printf(" Download from image offset "
+			fprintf(stderr, " Download from image offset "
 			       "%08x to memory %08x-%08x, size %i\n",
 			       p, address, address + chunk_size - 1,
 			       chunk_size);
@@ -467,7 +537,7 @@ dfuse_memcpy(unsigned char *dst, unsigned char **src, int *rem, int size)
 }
 
 /* Download raw binary file to DfuSe device */
-int dfuse_do_bin_dnload(struct dfu_if *dif, int xfer_size,
+static int dfuse_do_bin_dnload(struct dfu_if *dif, int xfer_size,
 			struct dfu_file *file, unsigned int start_address)
 {
 	unsigned int dwElementAddress;
@@ -479,7 +549,7 @@ int dfuse_do_bin_dnload(struct dfu_if *dif, int xfer_size,
 	dwElementSize = file->size.total -
 	    file->size.suffix - file->size.prefix;
 
-	printf("Downloading to address = 0x%08x, size = %i\n",
+	printf("Downloading element to address = 0x%08x, size = %i\n",
 	       dwElementAddress, dwElementSize);
 
 	data = file->firmware + file->size.prefix;
@@ -497,7 +567,7 @@ int dfuse_do_bin_dnload(struct dfu_if *dif, int xfer_size,
 }
 
 /* Parse a DfuSe file and download contents to device */
-int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
+static int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 			  struct dfu_file *file)
 {
 	uint8_t dfuprefix[11];
@@ -538,6 +608,9 @@ int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 	bTargets = dfuprefix[10];
 	printf("file contains %i DFU images\n", bTargets);
 
+	printf("Please note that the next version of dfu-util will automatically\n"
+	       "set alternate interfaces based on the DfuSe file images!\n");
+
 	for (image = 1; image <= bTargets; image++) {
 		printf("parsing DFU image %i\n", image);
 		dfuse_memcpy(targetprefix, &data, &rem, sizeof(targetprefix));
@@ -546,6 +619,10 @@ int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 			return -EINVAL;
 		}
 		bAlternateSetting = targetprefix[6];
+		if (targetprefix[7])
+			printf("Target name: %s\n", &targetprefix[11]);
+		else
+			printf("No target name\n");
 		dwNbElements = quad2uint((unsigned char *)targetprefix + 270);
 		printf("image for alternate setting %i, ", bAlternateSetting);
 		printf("(%i elements, ", dwNbElements);
@@ -592,7 +669,7 @@ int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 	if (rem != 0)
 		warnx("%d bytes leftover", rem);
 
-	printf("done parsing DfuSe file\n");
+	printf("Done parsing DfuSe file\n");
 
 	return 0;
 }
@@ -605,9 +682,11 @@ int dfuse_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file *file,
 	if (dfuse_options)
 		dfuse_parse_options(dfuse_options);
 	mem_layout = parse_memory_layout((char *)dif->alt_name);
-	if (!mem_layout) {
+	if (!mem_layout)
 		errx(EX_IOERR, "Failed to parse memory layout");
-	}
+	if (dif->quirks & QUIRK_DFUSE_LAYOUT)
+		fixup_dfuse_layout(dif, &mem_layout);
+
 	if (dfuse_unprotect) {
 		if (!dfuse_force) {
 			errx(EX_IOERR, "The read unprotect command "
@@ -626,7 +705,10 @@ int dfuse_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file *file,
 		printf("Performing mass erase, this can take a moment\n");
 		dfuse_special_command(dif, 0, MASS_ERASE);
 	}
-	if (dfuse_address) {
+	if (!file->name) {
+		printf("DfuSe command mode\n");
+		ret = 0;
+	} else if (dfuse_address_present) {
 		if (file->bcdDFU == 0x11a) {
 			errx(EX_IOERR, "This is a DfuSe file, not "
 				"meant for raw download");
@@ -642,10 +724,13 @@ int dfuse_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file *file,
 	}
 	free_segment_list(mem_layout);
 
-	dfu_abort_to_idle(dif);
+	if (!dfuse_will_reset) {
+		dfu_abort_to_idle(dif);
+	}
 
 	if (dfuse_leave) {
-		dfuse_special_command(dif, dfuse_address, SET_ADDRESS);
+		if (dfuse_address_present)
+			dfuse_special_command(dif, dfuse_address, SET_ADDRESS);
 		dfuse_dnload_chunk(dif, NULL, 0, 2); /* Zero-size */
 	}
 	return ret;
