@@ -21,15 +21,13 @@
 #ifndef DFUSE_H
 #define DFUSE_H
 
-#include <dfu.h>
+#include "dfu.h"
 
 enum dfuse_command { SET_ADDRESS, ERASE_PAGE, MASS_ERASE, READ_UNPROTECT };
 
-int dfuse_special_command(struct dfu_if *dif, unsigned int address,
-			  enum dfuse_command command);
-int dfuse_do_upload(struct dfu_if *dif, int xfer_size, struct dfu_file file,
+int dfuse_do_upload(struct dfu_if *dif, int xfer_size, int fd,
 		    const char *dfuse_options);
-int dfuse_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file file,
+int dfuse_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file *file,
 		    const char *dfuse_options);
 
 #endif /* DFUSE_H */
