@@ -2,9 +2,11 @@
  * dfu-util
  *
  * Copyright 2007-2008 by OpenMoko, Inc.
+ * Copyright 2010-2012 Stefan Schmidt
  * Copyright 2013-2014 Hans Petter Selasky <hps@bitfrost.no>
+ * Copyright 2010-2020 Tormod Volden
  *
- * Written by Harald Welte <laforge@openmoko.org>
+ * Originally written by Harald Welte <laforge@openmoko.org>
  *
  * Based on existing code of dfu-programmer-0.4
  *
@@ -30,26 +32,20 @@
 #include <libusb.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <limits.h>
 
 #include "portable.h"
 #include "dfu.h"
-#include "usb_dfu.h"
 #include "dfu_file.h"
 #include "dfu_load.h"
 #include "dfu_util.h"
 #include "dfuse.h"
-#include "quirks.h"
-
-#ifdef HAVE_USBPATH_H
-#include <usbpath.h>
-#endif
 
 int verbose = 0;
 
 struct dfu_if *dfu_root = NULL;
 
-int match_bus = -1;
-int match_device = -1;
+char *match_path = NULL;
 int match_vendor = -1;
 int match_product = -1;
 int match_vendor_dfu = -1;
@@ -57,6 +53,7 @@ int match_product_dfu = -1;
 int match_config_index = -1;
 int match_iface_index = -1;
 int match_iface_alt_index = -1;
+int match_devnum = -1;
 const char *match_iface_alt_name = NULL;
 const char *match_serial = NULL;
 const char *match_serial_dfu = NULL;
@@ -142,34 +139,26 @@ static void parse_serial(char *str)
 	if (*match_serial_dfu == 0) match_serial_dfu = NULL;
 }
 
-#ifdef HAVE_USBPATH_H
-
-static int resolve_device_path(char *path)
+static int parse_number(char *str, char *nmb)
 {
-	int res;
+	char *endptr;
+	long val;
 
-	res = usb_path2devnum(path);
-	if (res < 0)
-		return -EINVAL;
-	if (!res)
-		return 0;
+	errno = 0;
+	val = strtol(nmb, &endptr, 0);
 
-	match_bus = atoi(path);
-	match_device = res;
+	if ((errno == ERANGE && (val == LONG_MAX || val == LONG_MIN))
+		|| (errno != 0 && val == 0) || (*endptr != '\0')) {
+		errx(EX_SOFTWARE, "Something went wrong with the argument of --%s\n", str);
+	}
 
-	return 0;
+	if (endptr == nmb) {
+		errx(EX_SOFTWARE, "No digits were found from the argument of --%s\n", str);
+	}
+
+	return (int)val;
 }
 
-#else /* HAVE_USBPATH_H */
-
-static int resolve_device_path(char *path)
-{
-	(void)path; /* Eliminate unused variable warning */
-	errx(EX_SOFTWARE, "USB device paths are not supported by this dfu-util.\n");
-}
-
-#endif /* !HAVE_USBPATH_H */
-
 static void help(void)
 {
 	fprintf(stderr, "Usage: dfu-util [options] ...\n"
@@ -181,6 +170,7 @@ static void help(void)
 		"  -E --detach-delay seconds\tTime to wait before reopening a device after detach\n"
 		"  -d --device <vendor>:<product>[,<vendor_dfu>:<product_dfu>]\n"
 		"\t\t\t\tSpecify Vendor/Product ID(s) of DFU device\n"
+		"  -n --devnum <dnum>\t\tMatch given device number (devnum from --list)\n"
 		"  -p --path <bus-port. ... .port>\tSpecify path to DFU device\n"
 		"  -c --cfg <config_nr>\t\tSpecify the Configuration of DFU device\n"
 		"  -i --intf <intf_nr>\t\tSpecify the DFU Interface number\n"
@@ -193,9 +183,17 @@ static void help(void)
 		"  -Z --upload-size <bytes>\tSpecify the expected upload size in bytes\n"
 		"  -D --download <file>\t\tWrite firmware from <file> into device\n"
 		"  -R --reset\t\t\tIssue USB Reset signalling once we're finished\n"
-		"  -s --dfuse-address <address>\tST DfuSe mode, specify target address for\n"
-		"\t\t\t\traw file download or upload. Not applicable for\n"
-		"\t\t\t\tDfuSe file (.dfu) downloads\n"
+		"  -w --wait\t\t\tWait for device to appear\n"
+		"  -s --dfuse-address address<:...>\tST DfuSe mode string, specifying target\n"
+		"\t\t\t\taddress for raw file download or upload (not\n"
+		"\t\t\t\tapplicable for DfuSe file (.dfu) downloads).\n"
+		"\t\t\t\tAdd more DfuSe options separated with ':'\n"
+		"\t\tleave\t\tLeave DFU mode (jump to application)\n"
+		"\t\tmass-erase\tErase the whole device (requires \"force\")\n"
+		"\t\tunprotect\tErase read protected device (requires \"force\")\n"
+		"\t\twill-reset\tExpect device to reset (e.g. option bytes write)\n"
+		"\t\tforce\t\tYou really know what you are doing!\n"
+		"\t\t<length>\tLength of firmware to upload from device\n"
 		);
 	exit(EX_USAGE);
 }
@@ -204,7 +202,7 @@ static void print_version(void)
 {
 	printf(PACKAGE_STRING "\n\n");
 	printf("Copyright 2005-2009 Weston Schmidt, Harald Welte and OpenMoko Inc.\n"
-	       "Copyright 2010-2014 Tormod Volden and Stefan Schmidt\n"
+	       "Copyright 2010-2020 Tormod Volden and Stefan Schmidt\n"
 	       "This program is Free Software and has ABSOLUTELY NO WARRANTY\n"
 	       "Please report bugs to " PACKAGE_BUGREPORT "\n\n");
 }
@@ -231,6 +229,8 @@ static struct option opts[] = {
 	{ "download", 1, 0, 'D' },
 	{ "reset", 0, 0, 'R' },
 	{ "dfuse-address", 1, 0, 's' },
+	{ "devnum",1, 0, 'n' },
+	{ "wait", 1, 0, 'w' },
 	{ 0, 0, 0, 0 }
 };
 
@@ -244,6 +244,7 @@ int main(int argc, char **argv)
 	struct dfu_file file;
 	char *end;
 	int final_reset = 0;
+	int wait_device = 0;
 	int ret;
 	int dfuse_device = 0;
 	int fd;
@@ -259,7 +260,7 @@ int main(int argc, char **argv)
 
 	while (1) {
 		int c, option_index = 0;
-		c = getopt_long(argc, argv, "hVvleE:d:p:c:i:a:S:t:U:D:Rs:Z:", opts,
+		c = getopt_long(argc, argv, "hVvleE:d:p:c:i:a:S:t:U:D:Rs:Z:wn:", opts,
 				&option_index);
 		if (c == -1)
 			break;
@@ -279,30 +280,27 @@ int main(int argc, char **argv)
 			break;
 		case 'e':
 			mode = MODE_DETACH;
-			match_iface_alt_index = 0;
-			match_iface_index = 0;
 			break;
 		case 'E':
-			detach_delay = atoi(optarg);
+			detach_delay = parse_number("detach-delay", optarg);
 			break;
 		case 'd':
 			parse_vendprod(optarg);
 			break;
 		case 'p':
-			/* Parse device path */
-			ret = resolve_device_path(optarg);
-			if (ret < 0)
-				errx(EX_SOFTWARE, "Unable to parse '%s'", optarg);
-			if (!ret)
-				errx(EX_SOFTWARE, "Cannot find '%s'", optarg);
+#if (defined(LIBUSB_API_VERSION) && LIBUSB_API_VERSION >= 0x01000102) || (defined(LIBUSBX_API_VERSION) && LIBUSBX_API_VERSION >= 0x01000102)
+			match_path = optarg;
+#else
+			errx(EX_SOFTWARE, "This dfu-util was built without USB path support");
+#endif
 			break;
 		case 'c':
 			/* Configuration */
-			match_config_index = atoi(optarg);
+			match_config_index = parse_number("cfg", optarg);
 			break;
 		case 'i':
 			/* Interface */
-			match_iface_index = atoi(optarg);
+			match_iface_index = parse_number("intf", optarg);
 			break;
 		case 'a':
 			/* Interface Alternate Setting */
@@ -312,18 +310,21 @@ int main(int argc, char **argv)
 				match_iface_alt_index = -1;
 			}
 			break;
+		case 'n':
+			match_devnum = atoi(optarg);
+			break;
 		case 'S':
 			parse_serial(optarg);
 			break;
 		case 't':
-			transfer_size = atoi(optarg);
+			transfer_size = parse_number("transfer-size", optarg);
 			break;
 		case 'U':
 			mode = MODE_UPLOAD;
 			file.name = optarg;
 			break;
 		case 'Z':
-			expected_size = atoi(optarg);
+			expected_size = parse_number("upload-size", optarg);
 			break;
 		case 'D':
 			mode = MODE_DOWNLOAD;
@@ -335,18 +336,36 @@ int main(int argc, char **argv)
 		case 's':
 			dfuse_options = optarg;
 			break;
+		case 'w':
+			wait_device = 1;
+			break;
 		default:
 			help();
 			break;
 		}
 	}
+	if (optind != argc) {
+		fprintf(stderr, "Error: Unexpected argument: %s\n\n", argv[optind]);
+		help();
+	}
 
 	print_version();
 	if (mode == MODE_VERSION) {
 		exit(0);
 	}
 
-	if (mode == MODE_NONE) {
+#if defined(LIBUSB_API_VERSION) || defined(LIBUSBX_API_VERSION)
+	if (verbose) {
+		const struct libusb_version *ver;
+		ver = libusb_get_version();
+		printf("libusb version %i.%i.%i%s (%i)\n", ver->major,
+		       ver->minor, ver->micro, ver->rc, ver->nano);
+	}
+#else
+	warnx("libusb version is ancient");
+#endif
+
+	if (mode == MODE_NONE && !dfuse_options) {
 		fprintf(stderr, "You need to specify one of -D or -U\n");
 		help();
 	}
@@ -368,16 +387,29 @@ int main(int argc, char **argv)
 			match_product = file.idProduct;
 			printf("Match product ID from file: %04x\n", match_product);
 		}
+	} else if (mode == MODE_NONE && dfuse_options) {
+		/* for DfuSe special commands, match any device */
+		mode = MODE_DOWNLOAD;
+		file.idVendor = 0xffff;
+		file.idProduct = 0xffff;
+	}
+
+	if (wait_device) {
+		printf("Waiting for device, exit with ctrl-C\n");
 	}
 
 	ret = libusb_init(&ctx);
 	if (ret)
-		errx(EX_IOERR, "unable to initialize libusb: %i", ret);
+		errx(EX_IOERR, "unable to initialize libusb: %s", libusb_error_name(ret));
 
 	if (verbose > 2) {
+#if defined(LIBUSB_API_VERSION) && LIBUSB_API_VERSION >= 0x01000106
+		libusb_set_option(ctx, LIBUSB_OPTION_LOG_LEVEL, LIBUSB_LOG_LEVEL_DEBUG);
+#else
 		libusb_set_debug(ctx, 255);
+#endif
 	}
-
+probe:
 	probe_devices(ctx);
 
 	if (mode == MODE_LIST) {
@@ -386,7 +418,12 @@ int main(int argc, char **argv)
 	}
 
 	if (dfu_root == NULL) {
-		errx(EX_IOERR, "No DFU capable USB device available");
+		if (wait_device) {
+			milli_sleep(20);
+			goto probe;
+		} else {
+			errx(EX_IOERR, "No DFU capable USB device available");
+		}
 	} else if (dfu_root->next != NULL) {
 		/* We cannot safely support more than one DFU capable device
 		 * with same vendor/product ID, since during DFU we need to do
@@ -402,7 +439,7 @@ int main(int argc, char **argv)
 	printf("Opening DFU capable USB device...\n");
 	ret = libusb_open(dfu_root->dev, &dfu_root->dev_handle);
 	if (ret || !dfu_root->dev_handle)
-		errx(EX_IOERR, "Cannot open device");
+		errx(EX_IOERR, "Cannot open device: %s", libusb_error_name(ret));
 
 	printf("ID %04x:%04x\n", dfu_root->vendor, dfu_root->product);
 
@@ -421,13 +458,15 @@ int main(int argc, char **argv)
 		runtime_product = dfu_root->product;
 
 		printf("Claiming USB DFU Runtime Interface...\n");
-		if (libusb_claim_interface(dfu_root->dev_handle, dfu_root->interface) < 0) {
-			errx(EX_IOERR, "Cannot claim interface %d",
-				dfu_root->interface);
+		ret = libusb_claim_interface(dfu_root->dev_handle, dfu_root->interface);
+		if (ret < 0) {
+			errx(EX_IOERR, "Cannot claim interface %d: %s",
+				dfu_root->interface, libusb_error_name(ret));
 		}
 
-		if (libusb_set_interface_alt_setting(dfu_root->dev_handle, dfu_root->interface, 0) < 0) {
-			errx(EX_IOERR, "Cannot set alt interface zero");
+		ret = libusb_set_interface_alt_setting(dfu_root->dev_handle, dfu_root->interface, 0);
+		if (ret < 0) {
+			errx(EX_IOERR, "Cannot set alt interface zero: %s", libusb_error_name(ret));
 		}
 
 		printf("Determining device status: ");
@@ -463,7 +502,7 @@ int main(int argc, char **argv)
 				ret = libusb_reset_device(dfu_root->dev_handle);
 				if (ret < 0 && ret != LIBUSB_ERROR_NOT_FOUND)
 					errx(EX_IOERR, "error resetting "
-						"after detach");
+						"after detach: %s", libusb_error_name(ret));
 			}
 			break;
 		case DFU_STATE_dfuERROR:
@@ -529,24 +568,28 @@ int main(int argc, char **argv)
 dfustate:
 #if 0
 	printf("Setting Configuration %u...\n", dfu_root->configuration);
-	if (libusb_set_configuration(dfu_root->dev_handle, dfu_root->configuration) < 0) {
-		errx(EX_IOERR, "Cannot set configuration");
+	ret = libusb_set_configuration(dfu_root->dev_handle, dfu_root->configuration);
+	if (ret < 0) {
+		errx(EX_IOERR, "Cannot set configuration: %s", libusb_error_name(ret));
 	}
 #endif
 	printf("Claiming USB DFU Interface...\n");
-	if (libusb_claim_interface(dfu_root->dev_handle, dfu_root->interface) < 0) {
-		errx(EX_IOERR, "Cannot claim interface");
+	ret = libusb_claim_interface(dfu_root->dev_handle, dfu_root->interface);
+	if (ret < 0) {
+		errx(EX_IOERR, "Cannot claim interface - %s", libusb_error_name(ret));
 	}
 
 	printf("Setting Alternate Setting #%d ...\n", dfu_root->altsetting);
-	if (libusb_set_interface_alt_setting(dfu_root->dev_handle, dfu_root->interface, dfu_root->altsetting) < 0) {
-		errx(EX_IOERR, "Cannot set alternate interface");
+	ret = libusb_set_interface_alt_setting(dfu_root->dev_handle, dfu_root->interface, dfu_root->altsetting);
+	if (ret < 0) {
+		errx(EX_IOERR, "Cannot set alternate interface: %s", libusb_error_name(ret));
 	}
 
 status_again:
 	printf("Determining device status: ");
-	if (dfu_get_status(dfu_root, &status ) < 0) {
-		errx(EX_IOERR, "error get_status");
+	ret = dfu_get_status(dfu_root, &status );
+	if (ret < 0) {
+		errx(EX_IOERR, "error get_status: %s", libusb_error_name(ret));
 	}
 	printf("state = %s, status = %d\n",
 	       dfu_state_to_string(status.bState), status.bStatus);
@@ -599,17 +642,20 @@ status_again:
 
 	if (dfu_root->func_dfu.bcdDFUVersion == libusb_cpu_to_le16(0x11a))
 		dfuse_device = 1;
+	else if (dfuse_options)
+		printf("Warning: DfuSe option used on non-DfuSe device\n");
 
-	/* If not overridden by the user */
-	if (!transfer_size) {
-		transfer_size = libusb_le16_to_cpu(
-		    dfu_root->func_dfu.wTransferSize);
-		if (transfer_size) {
-			printf("Device returned transfer size %i\n",
-			       transfer_size);
-		} else {
+	/* Get from device or user, warn if overridden */
+	int func_dfu_transfer_size = libusb_le16_to_cpu(dfu_root->func_dfu.wTransferSize);
+	if (func_dfu_transfer_size) {
+		printf("Device returned transfer size %i\n", func_dfu_transfer_size);
+		if (!transfer_size)
+			transfer_size = func_dfu_transfer_size;
+		else
+			printf("Warning: Overriding device-reported transfer size\n");
+	} else {
+		if (!transfer_size)
 			errx(EX_IOERR, "Transfer size must be specified");
-		}
 	}
 
 #ifdef HAVE_GETPAGESIZE
@@ -687,7 +733,7 @@ status_again:
 		printf("Resetting USB to switch back to runtime mode\n");
 		ret = libusb_reset_device(dfu_root->dev_handle);
 		if (ret < 0 && ret != LIBUSB_ERROR_NOT_FOUND) {
-			errx(EX_IOERR, "error resetting after download");
+			errx(EX_IOERR, "error resetting after download: %s", libusb_error_name(ret));
 		}
 	}
 
